@@ -41,7 +41,8 @@ class overlay_interface(object):
 
     def __repr__(self):
         description = self.description or self.interface_id
-        return "(%s, %s)" % (self.node_id, description)
+#TODO: get the str of the node label rather than just node id
+        return "(%s, %s)" % (self.node, description)
 
     def __eq__(self, other):
         return self.__key() == other.__key()
@@ -80,7 +81,11 @@ class overlay_interface(object):
     @property
     def _interface(self):
         """Return data dict for the interface"""
-        return self._node["_interfaces"][self.interface_id]
+        try:
+            return self._node["_interfaces"][self.interface_id]
+        except KeyError:
+            log.warning("Unable to find interface %s in %s" % (self.interface_id, self.node))
+            return None
 
     @property
     def phy(self):
@@ -173,8 +178,9 @@ class overlay_interface(object):
         """Sets interface property"""
         try:
             self._interface[key] = val
-        except KeyError:
-            self.set(key, val)
+        except KeyError, e:
+            log.warning(e)
+            #self.set(key, val)
 
     def set(self, key, val):
         """For consistency, node.set(key, value) is neater
@@ -257,21 +263,26 @@ class OverlayNode(object):
 # so need to look at numeric part
         self_node_id = self.node_id
         other_node_id = other.node_id
-        self_node_string = [x for x in self.node_id if x not in string.digits]
-        other_node_string = [x for x in self.node_id if x not in string.digits]
-        if self_node_string == other_node_string:
-            self_node_id = "".join(
-                [x for x in self.node_id if x in string.digits])
-            other_node_id = "".join(
-                [x for x in other.node_id if x in string.digits])
-            try:
-                self_node_id = int(self_node_id)
-            except ValueError:
-                pass  # not a number
-            try:
-                other_node_id = int(other_node_id)
-            except ValueError:
-                pass  # not a number
+        try:
+            self_node_string = [x for x in self.node_id if x not in string.digits]
+            other_node_string = [x for x in self.node_id if x not in string.digits]
+        except TypeError:
+            # e.g. non-iterable type, such as an int node_id
+            pass
+        else:
+            if self_node_string == other_node_string:
+                self_node_id = "".join(
+                    [x for x in self.node_id if x in string.digits])
+                other_node_id = "".join(
+                    [x for x in other.node_id if x in string.digits])
+                try:
+                    self_node_id = int(self_node_id)
+                except ValueError:
+                    pass  # not a number
+                try:
+                    other_node_id = int(other_node_id)
+                except ValueError:
+                    pass  # not a number
 
         return ((self.asn, self_node_id) < (other.asn, other_node_id))
 
@@ -338,8 +349,15 @@ class OverlayNode(object):
                 return overlay_interface(self.anm, self.overlay_id, 
                         self.node_id, key.interface_id)
         except AttributeError:
-            log.warning("Unable to find interface %s in %s " % (key, self))
-            return None
+            #try with key as id
+            try:
+                if key in self._interface_ids:
+                    return overlay_interface(self.anm, self.overlay_id, 
+                            self.node_id, key)
+            except AttributeError:
+                # no match for either
+                log.warning("Unable to find interface %s in %s " % (key, self))
+                return None
 
     @property
     def _interface_ids(self):
@@ -402,7 +420,12 @@ class OverlayNode(object):
             return self._graph.node[self.node_id]['asn']  # not in this graph
         except KeyError:
             # try from phy
-            return self.anm.overlay_nx_graphs['phy'].node[self.node_id]['asn'] 
+            try:
+                return self.anm.overlay_nx_graphs['phy'].node[self.node_id]['asn'] 
+            except KeyError:
+                if self.node_id not in self.anm.overlay_nx_graphs['phy']:
+                    log.warning("Node id %s not found in physical overlay" % self.node_id)
+                    return
 
     @property
     def id(self):
@@ -454,6 +477,9 @@ class OverlayNode(object):
         """Edges to/from this node"""
         return self._overlay.edges(self, *args, **kwargs)
 
+    def __str__(self):
+        return str(self.__repr__())
+
     def __repr__(self):
         """Try label if set in overlay, otherwise from physical,
         otherwise node id"""
@@ -502,6 +528,15 @@ class OverlayEdge(object):
         object.__setattr__(self, 'src_id', src_id)
         object.__setattr__(self, 'dst_id', dst_id)
 
+    def __key(self):
+        # based on http://stackoverflow.com/q/2909106
+        """Note: key doesn't include overlay_id to allow fast cross-layer comparisons"""
+        return (self.src_id, self.dst_id)
+
+    def __hash__(self):
+        """"""
+        return hash(self.__key())
+
     def __eq__(self, other):
         """"""
         try:
@@ -678,7 +713,11 @@ class OverlayBase(object):
 
     def __contains__(self, n):
         """"""
-        return n.node_id in self._graph
+        try:
+            return n.node_id in self._graph
+        except AttributeError:
+            # try with node_id as a string
+            return n in self._graph
 
     def interface(self, interface):
         """"""
@@ -687,6 +726,16 @@ class OverlayBase(object):
 
     def edge(self, edge_to_find, dst_to_find=None):
         """returns edge in this graph with same src and same edge_id"""
+
+        if isinstance(edge_to_find, OverlayEdge):
+            src_id = edge_to_find.src
+            dst_id = edge_to_find.dst
+            #TODO: add MultiGraph support in terms of key here
+            for src, dst in self._graph.edges_iter(src_id):
+                if dst == dst_id:
+                    return OverlayEdge(self._anm, self._overlay_id, src, dst)
+
+        #TODO: tidy this logic up
         try:
             src = edge_to_find
             dst = dst_to_find
@@ -870,7 +919,6 @@ class OverlayBase(object):
                       for (src, dst) in valid_edges)
         return result
 
-
 class OverlaySubgraph(OverlayBase):
     """OverlaySubgraph"""
 
@@ -883,7 +931,6 @@ class OverlaySubgraph(OverlayBase):
     def __repr__(self):
         return self._subgraph_name or "subgraph"
 
-
 class OverlayGraph(OverlayBase):
     """API to interact with an overlay graph in ANM"""
 
@@ -983,27 +1030,68 @@ class OverlayGraph(OverlayBase):
                 log.debug("Initialise interfaces for %s in %s" % (
                     node, self._overlay_id))
                 self._graph.node[node]['_interfaces'] = {0:
-                                                         {'description': 'loopback',
-                                                             'type': 'loopback'}}
+                        {'description': 'loopback',
+                            'type': 'loopback'}}
 
     def allocate_interfaces(self):
         """allocates edges to interfaces"""
+
+        if self._overlay_id in ("input", "phy"):
+            if (all(len(node['input']._interfaces) > 0
+                for node in self)
+            and all(len(edge['input']._interfaces) > 0
+                for edge in self.edges()) ):
+                input_interfaces_allocated = True
+            else:
+                input_interfaces_allocated = False
+
+
+        if self._overlay_id == "input":
+            # only return if allocated here
+            if input_interfaces_allocated:
+                return # already allocated
+  
         # int_counter = (n for n in itertools.count() if n not in
+        if self._overlay_id == "phy":
+            # check if nodes added
+            nodes = list(self)
+            edges = list(self.edges())
+            if len(nodes) and len(edges):
+                # allocate called once physical graph populated
+
+                if input_interfaces_allocated:
+                    for node in self:
+                        input_interfaces = node['input']._interfaces
+                        if len(input_interfaces):
+                            node._interfaces = input_interfaces
+
+                    for edge in self.edges():
+                        edge._interfaces = edge['input']._interfaces
+                        input_interfaces = edge['input']._interfaces
+                        if len(input_interfaces):
+                            edge._interfaces = input_interfaces
+                    return
+
         self._init_interfaces()
 
+        #self._init_interfaces(nbunch)
+
         def numeric_id(edge):
             try:
                 return int(edge.edge_id.split("_")[0])
             except ValueError:
-                return edge # can't casr to int
+                return edge # can't cast to int
+            except AttributeError:
+                return edge # can't split, eg if not set
 
         ebunch = sorted(self.edges(), key = numeric_id)
 
         for edge in ebunch:
             src = edge.src
             dst = edge.dst
-            src_int_id = src._add_interface('%s to %s' % (src, dst))
-            dst_int_id = dst._add_interface('%s to %s' % (dst, src))
+            dst = edge.dst
+            src_int_id = src._add_interface('%s to %s' % (src.label, dst.label))
+            dst_int_id = dst._add_interface('%s to %s' % (dst.label, src.label))
             edge._interfaces = {}
             edge._interfaces[src.id] = src_int_id
             edge._interfaces[dst.id] = dst_int_id
@@ -1207,21 +1295,23 @@ class AbstractNetworkModel(object):
                 self._overlays[
                     overlay_id] = ank_json.ank_json_loads(graph_data)
 
+        ank_json.rebind_interfaces(self)
+
     @property
     def _phy(self):
         """"""
         return OverlayGraph(self, "phy")
 
     def initialise_graph(self, graph):
+        #TODO: check why this isn't used in build_network
         """Sets input graph. Converts to undirected. 
         Initialises graphics overlay."""
         graph = nx.Graph(graph)
-        g_in = self.add_overlay("input", graph=graph, directed=False)
         g_graphics = self['graphics']
+        g_in = self.add_overlay("input", graph=graph, directed=False)
         g_graphics.add_nodes_from(g_in, retain=['x', 'y', 'device_type',
                                                 'device_subtype', 'pop', 
                                                 'asn'])
-
         return g_in
 
     def add_overlay(self, name, nodes=None, graph=None, directed=False,
@@ -1249,6 +1339,7 @@ class AbstractNetworkModel(object):
         if nodes:
             retain = retain or []  # default is an empty list
             overlay.add_nodes_from(nodes, retain)
+
         return overlay
 
     def overlays(self):
