@@ -12,6 +12,7 @@ import autonetkit.ank as ank_utils
 import itertools
 
 #TODO: remove retain edge_id once removed from compiler
+#TODO: note that build network now assumes input graph has interface mappings on nodes/edges
 
 __all__ = ['build']
 
@@ -47,7 +48,7 @@ def grid_2d(dim):
         graph.node[n]['y'] = n[1] * 150
         graph.node[n]['device_type'] = 'router'
         graph.node[n]['platform'] = 'cisco'
-        graph.node[n]['syntax'] = 'ios2'
+        graph.node[n]['syntax'] = 'ios_xr'
         graph.node[n]['host'] = 'internal'
         graph.node[n]['ibgp_level'] = 0
 
@@ -95,8 +96,13 @@ def initialise(input_graph):
 
     g_graphics = anm.add_overlay("graphics")  # plotting data
     g_graphics.add_nodes_from(g_in, retain=['x', 'y', 'device_type',
-                              'device_subtype', 'pop', 'asn'])
+        'label', 'device_subtype', 'pop', 'asn'])
 
+    if g_in.data.Creator == "Maestro":
+        # Multiple ASNs set, use label format device.asn
+        anm.set_node_label(".", ['label_full'])
+
+    autonetkit.update_http(anm)
     return anm
 
 def apply_design_rules(anm):
@@ -105,6 +111,7 @@ def apply_design_rules(anm):
     build_phy(anm)
     g_phy = anm['phy']
 
+    autonetkit.update_http(anm)
     build_l3_connectivity(anm)
 
     build_vrf(anm) # need to do before to add loopbacks before ip allocations
@@ -317,7 +324,6 @@ def mark_ebgp_vrf(anm):
             edge.exclude = True # exclude from "regular" ebgp (as put into vrf stanza)
             edge.vrf = edge.dst['vrf'].vrf
 
-
 def build_vrf(anm):
     """Build VRF Overlay"""
     g_in = anm['input']
@@ -678,34 +684,42 @@ def build_ipv6(anm):
 def manual_ipv4_infrastructure_allocation(anm):
     """Applies manual IPv4 allocation"""
     import netaddr
-    g_in_directed = anm['input_directed']
     g_ipv4 = anm['ipv4']
-    #TODO: tidy this up to work with interfaces directly
 
-    for l3_device in g_ipv4.nodes("is_l3device"):
-        for edge in l3_device.edges():
-            # find edge in g_in_directed
-            directed_edge = g_in_directed.edge(edge)
-            edge.ip_address = netaddr.IPAddress(directed_edge.ipv4)
+    for node in g_ipv4.nodes("is_l3device"):
+        for interface in node.physical_interfaces:
+            if not interface['input'].is_bound:
+                continue # unbound interface
+            ip_address = netaddr.IPAddress(interface['input'].ipv4_address)
+            prefixlen = interface['input'].ipv4_prefixlen
+            interface.ip_address = ip_address
+            interface.prefixlen = prefixlen
+            cidr_string = "%s/%s" % (ip_address, prefixlen)
+            interface.subnet = netaddr.IPNetwork(cidr_string)
 
-            # set subnet onto collision domain (can come from either
-            # direction)
-            collision_domain = edge.dst
-            if not collision_domain.subnet:
-                # TODO: see if direct method in netaddr to deduce network
-                prefixlen = directed_edge.netPrefixLenV4
-                cidr_string = "%s/%s" % (edge.ip_address, prefixlen)
+    collision_domains = [d for d in g_ipv4 if d.collision_domain]
+    #TODO: allow this to work with specified ip_address/subnet as well as ip_address/prefixlen
+    from netaddr import IPNetwork
+    for cd in collision_domains:
+        connected_interfaces = [edge.dst_int for edge in cd.edges()]
+        cd_subnets = [IPNetwork("%s/%s" % (i.subnet.network, i.prefixlen))
+            for i in connected_interfaces]
 
-                intermediate_subnet = netaddr.IPNetwork(cidr_string)
-                cidr_string = "%s/%s" % (
-                    intermediate_subnet.network, prefixlen)
-                subnet = netaddr.IPNetwork(cidr_string)
-                collision_domain.subnet = subnet
+        try:
+            assert(len(set(cd_subnets)) == 1)
+        except AssertionError:
+            log.warning("Non matching subnets from collision domain %s" % cd)
+        else:
+            cd.subnet = cd_subnets[0] # take first entry
 
-    #TODO: assign directly to interfaces
+        # apply to remote interfaces
+        for edge in cd.edges():
+            remote_interface = edge.dst_int
+            edge.dst_int.subnet = cd.subnet
 
     # also need to form aggregated IP blocks (used for e.g. routing prefix
     # advertisement)
+    autonetkit.update_http(anm)
     infra_blocks = {}
     for asn, devices in g_ipv4.groupby("asn").items():
         collision_domains = [d for d in devices if d.collision_domain]
@@ -717,12 +731,10 @@ def manual_ipv4_infrastructure_allocation(anm):
 def manual_ipv4_loopback_allocation(anm):
     """Applies manual IPv4 allocation"""
     import netaddr
-    g_in_directed = anm['input_directed']
     g_ipv4 = anm['ipv4']
 
     for l3_device in g_ipv4.nodes("is_l3device"):
-        directed_node = g_in_directed.node(l3_device)
-        l3_device.loopback = directed_node.ipv4loopback
+        l3_device.loopback = l3_device['input'].loopback_v4
 
     # also need to form aggregated IP blocks (used for e.g. routing prefix
     # advertisement)
@@ -750,6 +762,7 @@ def build_ip(anm):
         "is_l3device")]
     for edge in edges_to_split:
         edge.split = True # mark as split for use in building nidb
+
     split_created_nodes = list(
         ank_utils.split(g_ip, edges_to_split, retain=['edge_id', 'split'], id_prepend = "cd"))
     for node in split_created_nodes:
@@ -832,8 +845,22 @@ def build_ipv4(anm, infrastructure=True):
     # check if ip ranges have been specified on g_in
     infra_block, loopback_block, vrf_loopback_block = extract_ipv4_blocks(anm)
 
+    # See if IP addresses specified on each interface
+    l3_devices = [d for d in g_in if d.device_type in ("router", "server")]
+
+    manual_alloc_devices = set()
+    for device in l3_devices:
+        physical_interfaces = list(device.physical_interfaces)
+        if all(interface.ipv4_address for interface in physical_interfaces):
+            manual_alloc_devices.add(device) # add as a manual allocated device
+
+    if manual_alloc_devices == set(l3_devices):
+        manual_alloc_ipv4_infrastructure = True
+    else:
+        manual_alloc_ipv4_infrastructure = False
+
     #TODO: need to set allocate_ipv4 by default in the readers
-    if g_in.data.alloc_ipv4_infrastructure is False:
+    if manual_alloc_ipv4_infrastructure:
         manual_ipv4_infrastructure_allocation(anm)
     else:
         ipv4.allocate_infra(g_ipv4, infra_block)
@@ -844,19 +871,12 @@ def build_ipv4(anm, infrastructure=True):
         ipv4.allocate_loopbacks(g_ipv4, loopback_block)
 
     #TODO: need to also support secondary_loopbacks for IPv6
+    #TODO: only call if secondaries are set
     ipv4.allocate_vrf_loopbacks(g_ipv4, vrf_loopback_block)
 
     #TODO: replace this with direct allocation to interfaces in ip alloc plugin
     for node in g_ipv4.nodes("is_l3device"):
         node.loopback_zero.ip_address = node.loopback
-        for interface in node:
-            edges = list(interface.edges())
-            if len(edges):
-                edge = edges[0] # first (only) edge
-                interface.ip_address = edge.ip_address
-                interface.subnet = edge.dst.subnet # from collision domain
-
-    # TODO: also map loopbacks to loopback interface 0
 
 def build_phy(anm):
     """Build physical overlay"""
@@ -877,7 +897,6 @@ def build_phy(anm):
 
     g_phy.add_edges_from(g_in.edges(type="physical"))
     # TODO: make this automatic if adding to the physical graph?
-    g_phy.allocate_interfaces() 
 
     if g_in.data.Creator == "Maestro":
         g_phy.data.mgmt_interfaces_enabled = g_in.data.mgmt_interfaces_enabled 
@@ -886,14 +905,10 @@ def build_phy(anm):
         g_phy.data.mgmt_prefixlen = g_in.data.mgmt_prefixlen
         ank_utils.copy_attr_from(g_in, g_phy, "use_cdp")
         ank_utils.copy_attr_from(g_in, g_phy, "use_onepk")
+        ank_utils.copy_attr_from(g_in, g_phy, "label_full")
+        ank_utils.copy_attr_from(g_in, g_phy, "indices")
 
-            # copy over numeric interface ids
-        for node in g_phy:
-            for interface in node:
-                edge = interface.edges()[0]
-                directed_edge = anm['input_directed'].edge(edge)
-                interface.numeric_int_id = directed_edge.numeric_int_id
-
+    g_phy.allocate_interfaces() 
     for node in g_phy.nodes("specified_int_names"):
         for interface in node:
             edge = interface.edges()[0]
@@ -919,7 +934,6 @@ def build_l3_connectivity(anm):
     for edge in exploded_edges:
         edge.multipoint = True
 
-
 def build_conn(anm):
     """Build connectivity overlay"""
     g_in = anm['input']
@@ -979,6 +993,8 @@ def build_ospf(anm):
         # string comparison as hasn't yet been cast to IPAddress
         default_area = area_zero_ip
 
+    #TODO: use interfaces throughout, rather than edges
+
     for router in g_ospf:
         if not router.area or router.area == "None":
             router.area = default_area
@@ -1078,7 +1094,6 @@ def ip_to_net_ent_title_ios(ip_addr):
     return ".".join([area_id, ip_octets[0:4], ip_octets[4:8], ip_octets[8:12],
                      "00"])
 
-
 def build_isis(anm):
     """Build isis overlay"""
     g_in = anm['input']
@@ -1111,8 +1126,6 @@ def build_isis(anm):
 
     for link in g_isis.edges():
         link.metric = 1  # default
-        # link.hello = 5 # for debugging, TODO: read from graph
-
 
     for edge in g_isis.edges():
         for interface in edge.interfaces():
