@@ -15,6 +15,23 @@ def dot_to_underscore(instring):
     """Replace dots with underscores"""
     return instring.replace(".", "_")
 
+def natural_sort(sequence, key=lambda s:s):
+    """
+    Sort the sequence into natural alphanumeric order.
+    """
+    import re
+    sequence = list(sequence)
+    def get_alphanum_key_func(key):
+        convert = lambda text: int(text) if text.isdigit() else text 
+        return lambda s: [convert(c) for c in re.split('([0-9]+)', key(s))]
+    sort_key = get_alphanum_key_func(key)
+    sequence.sort(key=sort_key)
+    return sequence
+
+def sort_sessions(sequence):
+    """Wrapper around natural_sort for bgp sessions"""
+    return natural_sort(sequence, key = lambda x: x.dst.label)
+
 class RouterCompiler(object):
     """Base router compiler"""
     lo_interface = "lo0"
@@ -120,32 +137,41 @@ class RouterCompiler(object):
                 continue
                 
             interface.description = phy_int.description
+            remote_edges = phy_int.edges()
+            if len(remote_edges):
+                interface.description = "to %s" % remote_edges[0].dst.label
+
+            #TODO: fix the description to use mapped label
             if node.ip.use_ipv4:
                 ipv4_int = phy_int['ipv4']
-                interface.use_ipv4 = True
-                interface.ipv4_address = ipv4_int.ip_address
-                interface.ipv4_subnet = ipv4_int.subnet
-                interface.ipv4_cidr = sn_preflen_to_network(interface.ipv4_address,
-                        interface.ipv4_subnet.prefixlen)
+                if ipv4_int.is_bound:
+                    # interface is connected
+                    interface.use_ipv4 = True
+                    interface.ipv4_address = ipv4_int.ip_address
+                    interface.ipv4_subnet = ipv4_int.subnet
+                    interface.ipv4_cidr = sn_preflen_to_network(interface.ipv4_address,
+                            interface.ipv4_subnet.prefixlen)
 
             if node.ip.use_ipv6:
                 ipv6_int = phy_int['ipv6']
-                interface.use_ipv6 = True
+                if ipv6_int.is_bound:
+                    # interface is connected
+                    interface.use_ipv6 = True
 #TODO: for consistency, make ipv6_cidr
-                interface.ipv6_subnet = ipv6_int.subnet
-                interface.ipv6_address = sn_preflen_to_network(ipv6_int.ip_address,
-                        interface.ipv6_subnet.prefixlen)
+                    interface.ipv6_subnet = ipv6_int.subnet
+                    interface.ipv6_address = sn_preflen_to_network(ipv6_int.ip_address,
+                            interface.ipv6_subnet.prefixlen)
 
         for interface in node.loopback_interfaces:
             #TODO: check if nonzero is different to __eq__
             if interface == node.loopback_zero:
                 continue
             else:
-                #print "here for non zero", interface.id
                 phy_int = self.anm['phy'].interface(interface)
                 if node.ip.use_ipv4:
                     ipv4_int = phy_int['ipv4']
                     interface.use_ipv4 = True
+
                     interface.ipv4_address = ipv4_int.loopback
                     interface.ipv4_subnet = node.loopback_subnet
                     interface.ipv4_cidr = sn_preflen_to_network(interface.ipv4_address,
@@ -228,7 +254,7 @@ class RouterCompiler(object):
         ibgp_rr_parents = []
 
         g_ibgp_v4 = self.anm['ibgp_v4']
-        for session in g_ibgp_v4.edges(phy_node):
+        for session in sort_sessions(g_ibgp_v4.edges(phy_node)):
             if session.exclude:
                 log.debug("Skipping excluded ibgp session %s" % session)
                 continue # exclude from regular ibgp config (eg for VRF, VPLS, etc)
@@ -245,7 +271,7 @@ class RouterCompiler(object):
 
         #TODO: check v6 hierarchy only created if node set to being v4 or v6
         g_ibgp_v6 = self.anm['ibgp_v6']
-        for session in g_ibgp_v6.edges(phy_node):
+        for session in sort_sessions(g_ibgp_v6.edges(phy_node)):
             if session.exclude:
                 log.debug("Skipping excluded ibgp session %s" % session)
                 continue # exclude from regular ibgp config (eg for VRF, VPLS, etc)
@@ -269,7 +295,7 @@ class RouterCompiler(object):
 
         ebgp_neighbors = []
         g_ebgp_v4 = self.anm['ebgp_v4']
-        for session in g_ebgp_v4.edges(phy_node):
+        for session in sort_sessions(g_ebgp_v4.edges(phy_node)):
             if session.exclude:
                 log.debug("Skipping excluded ebgp session %s" % session)
                 continue # exclude from regular ibgp config (eg for VRF, VPLS, etc)
@@ -277,7 +303,7 @@ class RouterCompiler(object):
             ebgp_neighbors.append(data)
 
         g_ebgp_v6 = self.anm['ebgp_v6']
-        for session in g_ebgp_v6.edges(phy_node):
+        for session in sort_sessions(g_ebgp_v6.edges(phy_node)):
             if session.exclude:
                 log.debug("Skipping excluded ebgp session %s" % session)
                 continue # exclude from regular ibgp config (eg for VRF, VPLS, etc)
@@ -291,7 +317,6 @@ class RouterCompiler(object):
 
     def isis(self, node):
         g_isis = self.anm['isis']
-        node.isis.isis_links = []
 
         for interface in node.physical_interfaces:
             if interface.exclude_igp:
@@ -310,12 +335,6 @@ class RouterCompiler(object):
                         'multipoint': isis_int.multipoint,
                         } #TODO: add wrapper for this
 
-                node.isis.isis_links.append(
-                        id = interface.id,
-                        metric = isis_int.metric,
-                        multipoint= isis_int.multipoint,
-                        )
-
         g_isis = self.anm['isis']
         isis_node = self.anm['isis'].node(node)
         node.isis.net = isis_node.net
@@ -404,9 +423,15 @@ class IosBaseCompiler(RouterCompiler):
         phy_node = self.anm['phy'].node(node)
 
         node.use_cdp = phy_node.use_cdp
-        if phy_node.device_subtype == "os":
+        if phy_node.device_subtype == "vios":
             # only copy across for certain reference platforms
             node.use_onepk = phy_node.use_onepk
+            node.no_service_config = True
+
+        if phy_node.device_subtype == "ultra":
+            # only copy across for certain reference platforms
+            node.transport_input_ssh_telnet = True
+            node.include_csr = True
 
         if node in self.anm['ospf']:
             node.ospf.use_ipv4 = phy_node.use_ipv4
@@ -471,13 +496,13 @@ class IosBaseCompiler(RouterCompiler):
             vrf_ibgp_neighbors = defaultdict(list)
 
             g_ibgp_v4 = self.anm['ibgp_v4']
-            for session in g_ibgp_v4.edges(vrf_node):
+            for session in sort_sessions(g_ibgp_v4.edges(vrf_node)):
                 if session.exclude and session.vrf:
                     data = self.ibgp_session_data(session, ip_version = 4)
                     vrf_ibgp_neighbors[session.vrf].append(data)
 
             g_ibgp_v6 = self.anm['ibgp_v6']
-            for session in g_ibgp_v6.edges(vrf_node):
+            for session in sort_sessions(g_ibgp_v6.edges(vrf_node)):
                 if session.exclude and session.vrf:
                     data = self.ibgp_session_data(session, ip_version = 6)
                     vrf_ibgp_neighbors[session.vrf].append(data)
@@ -486,13 +511,13 @@ class IosBaseCompiler(RouterCompiler):
             vrf_ebgp_neighbors = defaultdict(list)
 
             g_ebgp_v4 = self.anm['ebgp_v4']
-            for session in g_ebgp_v4.edges(vrf_node):
+            for session in sort_sessions(g_ebgp_v4.edges(vrf_node)):
                 if session.exclude and session.vrf:
                     data = self.ebgp_session_data(session, ip_version = 4)
                     vrf_ebgp_neighbors[session.vrf].append(data)
 
             g_ebgp_v6 = self.anm['ebgp_v6']
-            for session in g_ebgp_v6.edges(vrf_node):
+            for session in sort_sessions(g_ebgp_v6.edges(vrf_node)):
                 if session.exclude and session.vrf:
                     data = self.ebgp_session_data(session, ip_version = 6)
                     vrf_ebgp_neighbors[session.vrf].append(data)
@@ -593,8 +618,53 @@ class IosBaseCompiler(RouterCompiler):
                         'multipoint': ospf_int.multipoint,
                         } #TODO: add wrapper for this
 
-class IosClassicCompiler(IosBaseCompiler):
 
+    def isis(self, node):
+        super(IosBaseCompiler, self).isis(node)
+        for interface in node.physical_interfaces:
+            isis_int = self.anm['isis'].interface(interface)
+            edges = isis_int.edges()
+            if len(edges) != 1:
+                log.warning("Extended IOS config support not valid for multipoint ISIS connections")
+                continue
+                #TODO multipoint handling?
+            edge = edges[0]
+            dst = edge.dst
+            if not dst.is_router:
+                log.warning("Extended IOS config support not valid for non router ISIS connections")
+                continue
+
+            src_type = node.device_subtype
+            dst_type = dst['phy'].device_subtype
+            if src_type == "xrvr":
+                if dst_type == "vios":
+                    interface.isis.hello_padding_disable = True
+                elif dst_type == "ultra":
+                    interface.isis.hello_padding_disable = True
+                elif dst_type == "titanium":
+                    interface.isis.hello_padding_disable = True
+
+            if src_type == "vios":
+                if dst_type == "xrvr":
+                    interface.isis.mtu = 1430
+
+            if src_type == "ultra":
+                if dst_type == "xrvr":
+                    interface.isis.mtu = 1430
+
+            if src_type == "titanium":
+                if dst_type == "xrvr":
+                    interface.mtu = 1430 # for all of interface
+                    interface.isis.hello_padding_disable = True
+                elif dst_type == "vios":
+                    interface.isis.hello_padding_disable = True
+                elif dst_type == "ultra":
+                    interface.isis.hello_padding_disable = True
+
+            interface.isis_mtu = interface.isis.mtu
+            interface.hello_padding_disable = interface.isis.hello_padding_disable
+
+class IosClassicCompiler(IosBaseCompiler):
     def ospf(self, node):
         super(IosClassicCompiler, self).ospf(node)
         loopback_zero = node.loopback_zero
@@ -608,16 +678,9 @@ class IosClassicCompiler(IosBaseCompiler):
                         'multipoint': False,
                         } #TODO: add wrapper for this        
 
-
     def compile(self, node):
         super(IosClassicCompiler, self).compile(node)
 
-        phy_node = self.anm['phy'].node(node)
-        if phy_node.include_csr:
-            node.include_csr = True
-
-        node.platform_subtype = phy_node.device_subtype
-
     def bgp(self, node):
         super(IosClassicCompiler, self).bgp(node)
 
@@ -644,12 +707,13 @@ class IosClassicCompiler(IosBaseCompiler):
                 neigh_data = dict(neigh)
                 vpnv4_neighbors.append(neigh_data)
 
+        #vpnv4_neighbors = natural_sort(vpnv4_neighbors, key = lambda x: x['dst_int_ip'])
+        vpnv4_neighbors = sorted(vpnv4_neighbors, key = lambda x: x['loopback'])
         node.bgp.vpnv4_neighbors = vpnv4_neighbors
 
-class Ios2Compiler(IosBaseCompiler):
-
+class IosXrCompiler(IosBaseCompiler):
     def ospf(self, node):
-        super(Ios2Compiler, self).ospf(node)
+        super(IosXrCompiler, self).ospf(node)
         g_ospf = self.anm['ospf']
         interfaces_by_area = defaultdict(list)
 
@@ -679,6 +743,32 @@ class Ios2Compiler(IosBaseCompiler):
 
         node.ospf.interfaces = dict( interfaces_by_area)
 
+
+    def isis(self, node):
+        super(IosXrCompiler, self).isis(node)
+        node.isis.isis_links = []
+
+        for interface in node.physical_interfaces:
+            if interface.exclude_igp:
+                continue # don't configure IGP for this interface
+
+            #print interface.isis.dump()
+            # copy across attributes from the IosBaseCompiler setting step
+
+            isis_int = self.anm['isis'].interface(interface)
+            if isis_int and isis_int.is_bound:
+                data = {
+                        'id': interface.id,
+                        'metric': isis_int.metric,
+                        'multipoint': isis_int.multipoint,
+                        }
+                if interface.isis.hello_padding_disable is not None:
+                    data['hello_padding_disable'] = interface.isis.hello_padding_disable
+                if interface.isis.mtu is not None:
+                    data['mtu'] = interface.isis.hello_padding_disable
+
+                node.isis.isis_links.append(**data)
+
 class NxOsCompiler(IosBaseCompiler):
     def interfaces(self, node):
 # need to aggregate areas
@@ -763,6 +853,13 @@ class NetkitCompiler(PlatformCompiler):
             nidb_node.render.base_dst_folder = os.path.join("rendered", self.host, "netkit", folder_name)
             nidb_node.render.dst_file = "%s.startup" % folder_name
 
+            nidb_node.render.custom = {
+                    'abc': 'def.txt'
+
+
+
+                    }
+
 # allocate zebra information
             if nidb_node.is_router:
                 nidb_node.zebra.password = "1234"
@@ -862,11 +959,6 @@ class NetkitCompiler(PlatformCompiler):
 class CiscoCompiler(PlatformCompiler):
     """Platform compiler for Cisco"""
 
-    # def __init__(self, nidb, anm, host):
-# TODO: setup to remap allocate interface id function here
-        # super(CiscoCompiler, self).__init__(nidb, anm, host)
-
-
     @staticmethod
     def numeric_to_interface_label_ios(x):
         """Starts at GigabitEthernet0/1 """
@@ -885,9 +977,13 @@ class CiscoCompiler(PlatformCompiler):
         return "Ethernet2/%s" % x
 
     @staticmethod
-    def numeric_to_interface_label_ios2(x):
+    def numeric_to_interface_label_ios_xr(x):
         return "GigabitEthernet0/0/0/%s" % x
 
+    @staticmethod
+    def numeric_to_interface_label_linux(x):
+        return "eth%s" % x
+
     @staticmethod
     def loopback_interface_ids():
         for x in itertools.count(100): # start at 100 for secondary
@@ -901,7 +997,7 @@ class CiscoCompiler(PlatformCompiler):
             yield "GigabitEthernet0/%s" % x
 
     @staticmethod
-    def interface_ids_ra():
+    def interface_ids_ultra():
         #TODO: make this skip if in list of allocated ie [interface.name for interface in node]
         for x in itertools.count(0):
             yield "GigabitEthernet%s" % x
@@ -912,7 +1008,7 @@ class CiscoCompiler(PlatformCompiler):
             yield "Ethernet2/%s" % x
 
     @staticmethod
-    def interface_ids_ios2():
+    def interface_ids_ios_xr():
         for x in itertools.count(0):
             yield "GigabitEthernet0/0/0/%s" % x
 
@@ -942,17 +1038,18 @@ class CiscoCompiler(PlatformCompiler):
         else:
             dst_folder = os.path.join("rendered", self.host, "cisco")
 # TODO: merge common router code, so end up with three loops: routers, ios
-# routers, ios2 routers
+# routers, ios_xr routers
 
         # store autonetkit_cisco version
         from pkg_resources import get_distribution
         ank_cisco_version = get_distribution("autonetkit_cisco").version
 
-        for phy_node in g_phy.nodes('is_router', host=self.host):
+        for phy_node in g_phy.nodes('is_l3device', host=self.host):
             loopback_ids = self.loopback_interface_ids()
             # allocate loopbacks to routes (same for all ios variants)
             nidb_node = self.nidb.node(phy_node)
             nidb_node.ank_cisco_version = ank_cisco_version
+            nidb_node.indices = phy_node.indices
 
             for interface in nidb_node.loopback_interfaces:
                 if interface != nidb_node.loopback_zero:
@@ -961,14 +1058,43 @@ class CiscoCompiler(PlatformCompiler):
             # numeric ids
             numeric_int_ids = self.numeric_interface_ids()
             for interface in nidb_node.physical_interfaces:
-                phy_numeric_id = phy_node.interface(interface).numeric_int_id
+                phy_numeric_id = phy_node.interface(interface).numeric_id
                 if phy_numeric_id is None:
+                    #TODO: remove numeric ID code
                     interface.numeric_id = numeric_int_ids.next() 
                 else:
                     interface.numeric_id = int(phy_numeric_id) 
 
+        for phy_node in g_phy.nodes('is_server', host=self.host):
+            #TODO: look at server syntax also, same as for routers
+            nidb_node = self.nidb.node(phy_node)
+            for interface in nidb_node.physical_interfaces:
+                interface.id = self.numeric_to_interface_label_linux(interface.numeric_id)
+                nidb_node.ip.use_ipv4 = phy_node.use_ipv4
+                nidb_node.ip.use_ipv6 = phy_node.use_ipv6
+                phy_int = phy_node.interface(interface)
+
+                #TODO: make this part of the base device compiler, which server/router inherits
+                if nidb_node.ip.use_ipv4:
+                    ipv4_int = phy_int['ipv4']
+                    if ipv4_int.is_bound:
+                        # interface is connected
+                        interface.use_ipv4 = True
+                        interface.ipv4_address = ipv4_int.ip_address
+                        interface.ipv4_subnet = ipv4_int.subnet
+                        interface.ipv4_cidr = sn_preflen_to_network(interface.ipv4_address,
+                                interface.ipv4_subnet.prefixlen)
+                if nidb_node.ip.use_ipv6:
+                    ipv6_int = phy_int['ipv6']
+                    if ipv6_int.is_bound:
+                        # interface is connected
+                        interface.use_ipv6 = True
+#TODO: for consistency, make ipv6_cidr
+                        interface.ipv6_subnet = ipv6_int.subnet
+                        interface.ipv6_address = sn_preflen_to_network(ipv6_int.ip_address,
+                                interface.ipv6_subnet.prefixlen)
+
         for phy_node in g_phy.nodes('is_router', host=self.host, syntax='ios'):
-            specified_int_names = phy_node.specified_int_names
             nidb_node = self.nidb.node(phy_node)
             nidb_node.render.template = os.path.join("templates","ios.mako")
             if to_memory:
@@ -981,16 +1107,16 @@ class CiscoCompiler(PlatformCompiler):
             #TODO: write function that assigns interface number excluding those already taken
 
             # Assign interfaces
-            if phy_node.device_subtype == "os":
+            if phy_node.device_subtype == "vios":
                 int_ids = self.interface_ids_ios()
                 numeric_to_interface_label = self.numeric_to_interface_label_ios
-            elif phy_node.device_subtype == "ra":
-                int_ids = self.interface_ids_ra()
+            elif phy_node.device_subtype == "ultra":
+                int_ids = self.interface_ids_ultra()
                 numeric_to_interface_label = self.numeric_to_interface_label_ra
             else:
                 # default if no subtype specified
                 #TODO: need to set default in the load module
-                log.warning("Unexpected subtype %s" % phy_node.device_subtype)
+                log.warning("Unexpected subtype %s for %s" % (phy_node.device_subtype, phy_node))
                 int_ids = self.interface_ids_ios()
                 numeric_to_interface_label = self.numeric_to_interface_label_ios
                 
@@ -998,25 +1124,17 @@ class CiscoCompiler(PlatformCompiler):
                 mgmt_int_id = int_ids.next()  # 0/0 is used for management ethernet
 
             for interface in nidb_node.physical_interfaces:
-                # map numeric id to label
-                interface.label = numeric_to_interface_label(interface.numeric_id)
-
-                if specified_int_names:
-                    interface.id = phy_node.interface(interface).name
-                # TODO: need to determine if interface name already specified
-                else:
-                    interface.id = int_ids.next()
+                interface.id = numeric_to_interface_label(interface.numeric_id)
 
             ios_compiler.compile(nidb_node)
             if use_mgmt_interfaces:
                 mgmt_int = nidb_node.add_interface(management = True)
                 mgmt_int.id = mgmt_int_id
 
-        ios2_compiler = Ios2Compiler(self.nidb, self.anm)
-        for phy_node in g_phy.nodes('is_router', host=self.host, syntax='ios2'):
-            specified_int_names = phy_node.specified_int_names
+        ios_xr_compiler = IosXrCompiler(self.nidb, self.anm)
+        for phy_node in g_phy.nodes('is_router', host=self.host, syntax='ios_xr'):
             nidb_node = self.nidb.node(phy_node)
-            nidb_node.render.template = os.path.join("templates","ios2","router.conf.mako")
+            nidb_node.render.template = os.path.join("templates","ios_xr","router.conf.mako")
             if to_memory:
                 nidb_node.render.to_memory = True
             else:
@@ -1025,16 +1143,11 @@ class CiscoCompiler(PlatformCompiler):
                     phy_node)
 
             # Assign interfaces
-            int_ids = self.interface_ids_ios2()
+            int_ids = self.interface_ids_ios_xr()
             for interface in nidb_node.physical_interfaces:
-                interface.label = self.numeric_to_interface_label_ios2(interface.numeric_id)
-                if specified_int_names:
-                    interface.id = phy_node.interface(interface).name
-                # TODO: need to determine if interface name already specified
-                else:
-                    interface.id = int_ids.next()
+                interface.id = self.numeric_to_interface_label_ios_xr(interface.numeric_id)
 
-            ios2_compiler.compile(nidb_node)
+            ios_xr_compiler.compile(nidb_node)
 
             if use_mgmt_interfaces:
                 mgmt_int_id = "mgmteth0/0/CPU0/0"
@@ -1043,7 +1156,6 @@ class CiscoCompiler(PlatformCompiler):
 
         nxos_compiler = NxOsCompiler(self.nidb, self.anm)
         for phy_node in g_phy.nodes('is_router', host=self.host, syntax='nx_os'):
-            specified_int_names = phy_node.specified_int_names
             nidb_node = self.nidb.node(phy_node)
             nidb_node.render.template = os.path.join("templates","nx_os.mako")
             if to_memory:
@@ -1056,22 +1168,19 @@ class CiscoCompiler(PlatformCompiler):
             # Assign interfaces
             int_ids = self.interface_ids_nxos()
             for interface in nidb_node.physical_interfaces:
-                interface.label = self.numeric_to_interface_label_nxos(interface.numeric_id)
-                if specified_int_names:
-                    interface.id = phy_node.interface(interface).name
-                # TODO: need to determine if interface name already specified
-                else:
-                    interface.id = int_ids.next()
+                interface.id = self.numeric_to_interface_label_nxos(interface.numeric_id)
+
+            nxos_compiler.compile(nidb_node)
+            #TODO: make this work other way around
 
             if use_mgmt_interfaces:
                 mgmt_int_id = "mgmt0"
                 mgmt_int = nidb_node.add_interface(management = True)
                 mgmt_int.id = mgmt_int_id
 
-            nxos_compiler.compile(nidb_node)
 
         other_nodes = [phy_node for phy_node in g_phy.nodes('is_router', host=self.host)
-                       if phy_node.syntax not in ("ios", "ios2")]
+                       if phy_node.syntax not in ("ios", "ios_xr")]
         for node in other_nodes:
             #TODO: check why we need this
             phy_node = g_phy.node(node)
@@ -1115,7 +1224,7 @@ class CiscoCompiler(PlatformCompiler):
 
         mgmt_subnet = start_subnet
         hosts_to_allocate = sorted(self.nidb.nodes('is_router', host=self.host))
-        dhcp_subtypes = {"os"}
+        dhcp_subtypes = {"vios"}
         dhcp_hosts = [h for h in hosts_to_allocate if h.device_subtype in dhcp_subtypes]
         non_dhcp_hosts = [h for h in hosts_to_allocate if h.device_subtype not in dhcp_subtypes]
 
@@ -1137,6 +1246,8 @@ class CiscoCompiler(PlatformCompiler):
                     interface.mgmt = True
                     if nidb_node.ip.use_ipv4:
                         interface.use_ipv4 = True
+                    if nidb_node.use_cdp:
+                        interface.use_cdp = True # ensure CDP activated
                     if nidb_node in dhcp_hosts:
                         interface.use_dhcp = True
                         oob_management_ips[str(nidb_node)] = "dhcp"
